package com.steerinfo.dil.controller;

import com.steerinfo.dil.model.TmstruckLoadResult;
import com.steerinfo.dil.service.ITmstruckLoadResultService;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.dil.util.PageListAdd;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;

/**
 * TmstruckLoadResult RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-02 10:38
 * 类描述
 * 修订历史：
 * 日期：2021-09-02
 * 作者：generator
 * 参考：
 * 描述：TmstruckLoadResult RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/tmstruckloadresults")
public class TmstruckLoadResultController extends BaseRESTfulController {

    @Autowired
    ITmstruckLoadResultService tmstruckLoadResultService;

    @Autowired
    ColumnDataUtil columnDataUtil;

    @ApiOperation(value="查询所有装车实绩")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mapValue", value = "表头和参数", required = false, dataType = "map"),
            @ApiImplicitParam(name = "apiId(91)", value = "动态表头", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageNum", value = "页码", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页", required = false, dataType = "Integer"),
    })
    @PostMapping("/getAllLoadResult")
    public RESTfulResult getAllLoadResult(@RequestBody(required=false) Map<String,Object> mapValue,
                                        Integer apiId,
                                        Integer pageNum,
                                        Integer pageSize,
                                          Integer status ){
        if(mapValue == null){
            mapValue = new HashMap<>();
        }
        if(status != null){
            mapValue.put("status", status);
        }
        //不分页筛选数据
        List<Map<String, Object>> allLoadResult = tmstruckLoadResultService.getAllLoadResult(mapValue);
        PageHelper.startPage(pageNum,pageSize);
        //分页数据
        List<Map<String, Object>> loadResult = tmstruckLoadResultService.getAllLoadResult(mapValue);
        DataChange.changeDateToDayDate(loadResult, "resultLoadStartTime");
        PageListAdd pageList = columnDataUtil.tableColumnData(apiId, allLoadResult,loadResult);
        return success(pageList);
    }

    @ApiOperation(value="新增汽车装车实绩")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mapValue", value = "", required = false, dataType = "Map"),
    })
    @PostMapping("/addLoadResult")
    public RESTfulResult addLoadResult(@RequestBody(required=false) Map<String,Object> mapValue){
        int i = tmstruckLoadResultService.addLoadResult(mapValue);
        return success(i);
    }

    @ApiOperation(value="修改汽车装车实绩")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tmstruckLoadResult", value = "装车作业实绩对象", required = false, dataType = "TmstruckLoadResult"),
    })
    @PostMapping("/updateLoadResult")
    public RESTfulResult updateLoadResult(@RequestBody TmstruckLoadResult tmstruckLoadResult){
        tmstruckLoadResult.setUpdateUsername("admin");
        tmstruckLoadResult.setUpdateTime(new Date());
        int i = tmstruckLoadResultService.updateLoadResult(tmstruckLoadResult);
        return success(i);
    }

    @ApiOperation(value="通过ID查询装车实绩 ")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "resultId", value = "装车作业实绩ID", required = false, dataType = "Integer"),
    })
    @PostMapping("/getLoadResultById/{resultId}")
    public RESTfulResult getLoadResultById(@PathVariable("resultId") Integer resultId){
        List<Map<String, Object>> date = tmstruckLoadResultService.getLoadResultById(resultId);
        return success(date);
    }

    @ApiOperation(value="逻辑删除车装车实绩")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "tmstruckLoadResult", value = "装车作业实绩对象", required = false, dataType = "TmstruckLoadResult"),
    })
    @PostMapping("/deleteLoadResult/{resultId}")
    public RESTfulResult deleteLoadResult(@PathVariable("resultId")Integer resultId){
        TmstruckLoadResult tmstruckLoadResult = new TmstruckLoadResult();
        tmstruckLoadResult.setDeleteName("admin");
        tmstruckLoadResult.setDeleteTime(new Date());
        tmstruckLoadResult.setResultId(new BigDecimal(resultId));
        tmstruckLoadResult.setStatus(new BigDecimal(1));
        int i = tmstruckLoadResultService.updateLoadResult(tmstruckLoadResult);
        return success(i);
    }
}
