package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.AmstruckPurplanMaterialMapper;
import com.steerinfo.dil.mapper.OmstruckOrderMapper;
import com.steerinfo.dil.mapper.OmstruckOrderMaterialMapper;
import com.steerinfo.dil.mapper.TmstruckTotalResultMapper;
import com.steerinfo.dil.model.*;
import com.steerinfo.dil.service.IOmstruckOrderService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * OmstruckOrder服务实现：
 * @author TXF
 * @version 1.0-SNAPSHORT 2021-09-03 03:47
 * 类描述
 * 修订历史：
 * 日期：2021-09-03
 * 作者：generator
 * 参考：
 * 描述：OmstruckOrder服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "omstruckOrderService")
public class OmstruckOrderServiceImpl implements IOmstruckOrderService {

    @Autowired
    private OmstruckOrderMapper omstruckOrderMapper;

    @Autowired
    private AmstruckPurplanMaterialMapper amstruckPurplanMaterialMapper;

    @Autowired
    private OmstruckOrderMaterialMapper omstruckOrderMaterialMapper;

    @Autowired
    private TmstruckTotalResultMapper tmstruckTotalResultMapper;

    @Autowired
    private TmstruckEnfactoryResultServiceImpl tmstruckEnfactoryResultService;
    @Autowired
    private TmstruckLeaveFactoryResultServiceImpl tmstruckLeaveFactoryResultService;
    @Autowired
    private TmstruckWeightResultServiceImpl tmstruckWeightResultService;
    @Autowired
    private TmstruckUnloadResultServiceImpl tmstruckUnloadResultService;
    /**
     * 查询所有运输订单
     * 获取已下发的运输订单 用于websocket
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllTruckOrder(Map<String, Object> map) {
        return omstruckOrderMapper.getAllTruckOrder(map);
    }

    /**
     * 生成运输订单实体类
     */
    public OmstruckOrder generateOmsTruckOrder(Map<String, Object> map){
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        //生成订单主键Id
        BigDecimal maxId = omstruckOrderMapper.selectMaxId();
        omstruckOrder.setOrderId(maxId);
        //生成运输订单号
        int id = maxId.intValue();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String orderNumber = "WYSDD" + sdf.format(new Date());
        orderNumber +=
                id < 10 ? "0000000" + id : id < 100 ? "000000" + id : id < 1000 ? "00000" + id : id < 10000 ?
                        "0000" + id : id < 100000 ? "000" + id : id < 1000000 ? "00" + id : id < 10000000 ? "0" + id : id;
        omstruckOrder.setOrderNumber(orderNumber);
        //运单状态 刚分配默认为未下发
        omstruckOrder.setOrderStatus(new BigDecimal(3));
        //添加采购订单ID
        omstruckOrder.setOrderPlanId(DataChange.dataToBigDecimal(map.get("purchaseOrderId")));
        //添加运力Id
        BigDecimal capacityId = DataChange.dataToBigDecimal(map.get("capacityId"));
        omstruckOrder.setCapacityId(capacityId);
        //添加订单类型 3
        omstruckOrder.setOrderType(new BigDecimal(3));

        omstruckOrder.setInsertUsername("admin");
        omstruckOrder.setInsertTime(new Date());
        return omstruckOrder;
    }

    /**
     * 新增采购 运输订单
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int addPurOrder(Map<String, Object> map) {
        //生成运输订单号和主键
        OmstruckOrder omstruckOrder = generateOmsTruckOrder(map);
        //设置线路  遗留问题
        omstruckOrder.setLineId(new BigDecimal(1));
        Object orderEntryTime = map.get("orderEntryTime");
        if(orderEntryTime != null){
            omstruckOrder.setOrderEntryTime(new Date((long) orderEntryTime)); //新增预计进厂时间
        }
        if(map.get("orderMaterialWeight") != null){
            updateMaterial(map); //更改运输计划子表 增加已分配重量 减少未分配重量
        }
        //新增运输订单
        omstruckOrderMapper.insertSelective(omstruckOrder);
        //更新运力状态为待运输：2
        map.put("capacityStatus", 2);
        omstruckOrderMapper.updateCapacityByCarIdOrNum(map);
        //新增运输订单子表
        addOrderMaterial(map, omstruckOrder.getOrderId());
        //新增实绩总表关联订单
        addTotalResult(omstruckOrder.getOrderId());
        return 5;
    }


    /**
     * 新增实绩总表
     * 运输订单ID
     * @param orderId
     * @return
     */
    public int addTotalResult(BigDecimal orderId){
        TmstruckTotalResult tmstruckTotalResult = new TmstruckTotalResult();
        tmstruckTotalResult.setResultTotalId(tmstruckTotalResultMapper.selectMaxId());
        tmstruckTotalResult.setOrderId(orderId);
        tmstruckTotalResult.setInsertTime(new Date());
        tmstruckTotalResult.setInsertUsername("admin");
        return tmstruckTotalResultMapper.insertSelective(tmstruckTotalResult);
    }

    /**
     * 新增订单子表
     * @param map
     * @param
     */
    public void addOrderMaterial(Map<String, Object> map, BigDecimal orderId){
        //新增运输订单子表
        OmstruckOrderMaterial omstruckOrderMaterial = new OmstruckOrderMaterial();
        //生成子表主键
        omstruckOrderMaterial.setOrderMaterialId(omstruckOrderMaterialMapper.selectMaxId());
        omstruckOrderMaterial.setMaterialId(DataChange.dataToBigDecimal(map.get("materialId")));
        omstruckOrderMaterial.setOrderId(orderId);
        Object orderMaterialWeight = map.get("orderMaterialWeight"); // 采购辅料没有物资重量
        if(orderMaterialWeight != null){
            omstruckOrderMaterial.setOrderMaterialWeight(DataChange.dataToBigDecimal(orderMaterialWeight));
        }
        omstruckOrderMaterial.setInsertUsername("admin");
        omstruckOrderMaterial.setInsertTime(new Date());
        omstruckOrderMaterialMapper.insertSelective(omstruckOrderMaterial);
    }

    /**
     * 用于更改计划子表方法
     * @param map
     */
    public void updateMaterial(Map<String, Object> map){
        Object omw = map.get("orderMaterialWeight");
        double weight;
        if(omw instanceof Double){
            weight = (Double) omw;
        }else if(omw instanceof Integer){
            weight = (Integer) omw;
        }else {
            weight = Double.parseDouble(omw.toString());
        }
        Integer planId = DataChange.dataToBigDecimal(map.get("planId")).intValue();
        List<Map<String, Object>> weightList = amstruckPurplanMaterialMapper.getWeight(planId);
        Map<String, Object> weightMap = weightList.get(0);
        double materialAllocatedWeight = DataChange.dataToBigDecimal(weightMap.get("materialAllocatedWeight")).floatValue();
        double materialUnallocatedWeight = DataChange.dataToBigDecimal(weightMap.get("materialUnallocatedWeight")).floatValue();
        //float allWeight = materialAllocatedWeight + materialUnallocatedWeight;
        //前端判断数据分配重量是否大于未分配重量 此处不做判断

        //通过前端是否传来旧的重量 判断 是否是在原有已分解过的计划上修改 未分配 已分配重量
        if(map.get("weight") != null){
            BigDecimal oldWeight = DataChange.dataToBigDecimal(map.get("weight"));
            double difference = weight - oldWeight.doubleValue();
            materialAllocatedWeight += difference;
            materialUnallocatedWeight -= difference;
        }else {
            materialAllocatedWeight += weight;
            materialUnallocatedWeight -= weight;
        }
        AmstruckPurplanMaterial amstruckPurplanMaterial = new AmstruckPurplanMaterial();
        //通过计划Id查询子表主键
        amstruckPurplanMaterial.setPurplanPlanMaterialId(amstruckPurplanMaterialMapper.getChildIdByPlanId(new BigDecimal(planId)));
        amstruckPurplanMaterial.setMaterialAllocatedWeight(new BigDecimal(materialAllocatedWeight));
        amstruckPurplanMaterial.setMaterialUnallocatedWeight(new BigDecimal(materialUnallocatedWeight));
        amstruckPurplanMaterialMapper.updateByPrimaryKeySelective(amstruckPurplanMaterial);
    }

    /**
     * 查询所有运力信息
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllCapacity(Map<String, Object> map) {
        return omstruckOrderMapper.getAllCapacity(map);
    }

    /**
     * 修改运输订单 和订单子表
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int updateOrder(Map<String, Object> map) {
        //修改运输订单运力ID
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        BigDecimal orderId = DataChange.dataToBigDecimal(map.get("orderId"));
        omstruckOrder.setOrderId(orderId);
        if(map.get("capacityId") != null){
            omstruckOrder.setCapacityId(DataChange.dataToBigDecimal(map.get("capacityId")));
        }
        if(map.get("orderEntryTime") != null){
            omstruckOrder.setOrderEntryTime(new Date((long) map.get("orderEntryTime")));
        }
        omstruckOrderMapper.updateByPrimaryKeySelective(omstruckOrder);

        //修改运输订单子表物资重量
        OmstruckOrderMaterial omstruckOrderMaterial = new OmstruckOrderMaterial();
        //添加子表主键 通过订单ID查询出子表主键
        BigDecimal orderMaterialId = omstruckOrderMaterialMapper.getOrderMaterialIdByOrderId(orderId);
        omstruckOrderMaterial.setOrderMaterialId(orderMaterialId);
        if(map.get("orderMaterialWeight") != null){
            omstruckOrderMaterial.setOrderMaterialWeight(DataChange.dataToBigDecimal(map.get("orderMaterialWeight")));
            //修改运输计划子表 已分配未分配重量
            updateMaterial(map);
        }
        if(map.get("materialId") != null){
            omstruckOrderMaterial.setMaterialId(DataChange.dataToBigDecimal(map.get("materialId")));
        }
        omstruckOrderMaterialMapper.updateByPrimaryKeySelective(omstruckOrderMaterial);

        return 1;
    }

    /**
     * 运单派发
     *  运单ID
     * 生成进厂实绩、计量实绩、卸货实绩、出厂实绩
     * @param orderId
     * @return
     */
    @Override
    @Transactional
    public int dispatchOrder(Integer orderId) {
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        omstruckOrder.setOrderId(new BigDecimal(orderId));
        //设置状态为已下发：4
        //生成订单下发时间 修改订单表
        omstruckOrder.setOrderIssueTime(new Date());
        omstruckOrder.setOrderStatus(new BigDecimal(4));
        omstruckOrderMapper.updateByPrimaryKeySelective(omstruckOrder);
        //通过运输订单ID查询运输订单号 （使用的都是运输订单号）
        String orderNumber = omstruckOrderMapper.getOrderNumberByOrderId(orderId);
        //通过运输订单号 查询各个作业点ID 及 总实绩ID
        Map<String, Object> map = omstruckOrderMapper.getPointIdByOrderNum(orderNumber);
        tmstruckEnfactoryResultService.addEnFactoryResult(map);
        tmstruckWeightResultService.addWeightResult(map);
        tmstruckLeaveFactoryResultService.addLeaveFactory(map);
        tmstruckUnloadResultService.addUnloadResult(map);
        return 5;
    }

    /**
     * 逻辑删除未下发订单
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int deleteOrder(Map<String, Object> map) {
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        omstruckOrder.setOrderId(DataChange.dataToBigDecimal(map.get("orderId")));
        //设置状态为逻辑删除：7
        omstruckOrder.setOrderStatus(new BigDecimal(7));
        int i = omstruckOrderMapper.updateByPrimaryKeySelective(omstruckOrder);
        //若是传来物资重量则修改已分配未分配重量
        if(map.get("orderMaterialWeight") != null){
            updateMaterial(map);
        }
        //通过车牌号修改运输状态为空闲中 0
        String capacityNumber = (String) map.get("capacityNumber");
        HashMap<String, Object> changeStatusMap = new HashMap<>();
        changeStatusMap.put("capacityNumber", capacityNumber);
        changeStatusMap.put("capacityStatus", 0);
        int i1 = omstruckOrderMapper.updateCapacityByCarIdOrNum(changeStatusMap);
        return i + i1;
    }

    /**
     * 司机接单、拒绝 0：拒绝   1：接收
     * 需要接收参数 主键Id 与 接收拒绝操作码
     * @param map
     * @return
     */
    @Override
    public int driverReceiveOrRefuse(Map<String, Object> map) {
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        if((Integer) map.get("orderReceiveStatus") == 1){
            omstruckOrder.setOrderReceiveStatus(new BigDecimal(1));
            omstruckOrder.setOrderStatus(new BigDecimal(5)); //5 ：已接收
        }else { //其他状态都视为 拒绝接单
            omstruckOrder.setOrderReceiveStatus(new BigDecimal(0));
            omstruckOrder.setOrderStatus(new BigDecimal(6));// 6：未接收
            //将司机运力置为空闲 通过运输订单Id 查询运力ID
            BigDecimal capacityId = omstruckOrderMapper.getCapacityIdByOrderOrNum(map);
            map.put("capacityId", capacityId);
            map.put("capacityStatus", 0);
            //修改运力表
            omstruckOrderMapper.updateCapacityByCarIdOrNum(map);
        }
        BigDecimal orderId = DataChange.dataToBigDecimal(map.get("orderId"));
        omstruckOrder.setOrderId(orderId);
        //生成接收/拒绝时间
        Date receiveTime = new Date();
        omstruckOrder.setOrderReceiveRefuseTime(receiveTime);
        //计算司机从派单到操作时长
        Date issueTime = omstruckOrderMapper.getInsertTimeById(orderId.intValue());
        long time = receiveTime.getTime() - issueTime.getTime();
        long min = time / (1000 * 60);
        omstruckOrder.setOrderCommunicationDuration(new BigDecimal(min));

        return  omstruckOrderMapper.updateByPrimaryKeySelective(omstruckOrder);
    }

    /**
     * 用于后台展示司机接单未接单数据
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getReceiveRefuseOrder(Map<String, Object> map) {
        return omstruckOrderMapper.getTruckOrderByCapacityNum(map);
    }

    /**
     * 司机接单推送数据
     * 车牌号
     * @param
     * @return
     */
    @Override
    public List<Map<String, Object>> sendMesToDriver(Map<String, Object> map) {

        //通过车牌号匹配运力ID
        BigDecimal capacityId = omstruckOrderMapper.getCapacityIdByCapacityNum((String) map.get("capacityNumber"));
        map.put("capacityId", capacityId);
        if(capacityId == null)
        return null;
        //通过运力Id匹配 已下发的运输订单
        List<Map<String, Object>> truckOrderByCapacityNum = omstruckOrderMapper.getTruckOrderByCapacityNum(map);
        return truckOrderByCapacityNum;
    }

    /**
     * 通过运输订单号查询运单所有信息 包含作业路径名称
     *
     * @param map 传运输订单号
     * @return
     */
    @Override
    public List<Map<String, Object>> getOrderMesByOrderNum(Map<String, Object> map) {
        return omstruckOrderMapper.getTruckOrderByCapacityNum(map);
    }

    /**
     * 查询所有司机已拒绝的订单
     * 车牌号
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getReceiveRefuseOrderByCapacityId(Map<String, Object> map) {
        //通过车牌号查询运力Id
        BigDecimal capacityId = omstruckOrderMapper.getCapacityIdByCapacityNum((String) map.get("capacityNumber"));
        map.put("capacityId", capacityId);

        return  omstruckOrderMapper.getReceiveRefuseOrderByCapacityId(map);
    }

    /**
     * 得到作业时间和地点
     * @param orderId
     * @return
     */
    @Override
    public List<Map<String, Object>> selectPlaceAndTime(BigDecimal orderId) {
        List<Map<String, Object>> mapList = omstruckOrderMapper.selectPlaceAndTime(orderId);
        Map<String, Object> map = mapList.get(0);
        String entryGatepost = (String) map.get("entryGatepost");
        Date entryTime = (Date) map.get("entryTime");
        String entryResult = entryGatepost + "/" + entryTime;
        String grossCalculate = (String) map.get("grossCalculate");
        Date grossTime = (Date) map.get("grossTime");
        String grossResult = grossCalculate + "/" + grossTime;
        String unloadWarhouse = (String) map.get("unloadWarhouse");
        Date unloadTime = (Date) map.get("unloadTime");
        String unloadResult = unloadWarhouse + "/" + unloadTime;
        String tareCalculate = (String) map.get("tareCalculate");
        Date tareTime = (Date) map.get("tareTime");
        String tareResult = tareCalculate + "/" + tareTime;
        String leaveGatepost = (String) map.get("leaveGatepost");
        Date outTime = (Date) map.get("outTime");
        String leaveResult = leaveGatepost + "/" + outTime;
        List<Map<String, Object>> resultList = new ArrayList<>();
        Map<String, Object> map1 = new HashMap<>();
        map1.put("title","进厂门岗/时间");
        map1.put("desc",entryResult);
        resultList.add(map1);
        Map<String, Object> map2 = new HashMap<>();
        map2.put("title","毛重衡器/时间");
        map2.put("desc",grossResult);
        resultList.add(map2);
        Map<String, Object> map3 = new HashMap<>();
        map3.put("title","卸货地点/时间");
        map3.put("desc",unloadResult);
        resultList.add(map3);
        Map<String, Object> map4 = new HashMap<>();
        map4.put("title","皮重衡器/时间");
        map4.put("desc",tareResult);
        resultList.add(map4);
        Map<String, Object> map5 = new HashMap<>();
        map5.put("title","出厂门岗/时间");
        map5.put("desc",leaveResult);
        resultList.add(map5);
        Map<String, Object> map6 = new HashMap<>();
        BigDecimal orderLineSequence = (BigDecimal) map.get("orderLineSequence");
        map6.put("orderLineSequence",orderLineSequence);
        resultList.add(map6);
        return resultList;
    }

    /**
     * 通过运单号查询订单详情
     * 运输订单号
     * @param
     * @return
     */
    @Override
    public List<Map<String, Object>> getOrderByOrderId(BigDecimal orderId) {

        return omstruckOrderMapper.getOrderByOrderId(orderId);
    }

    /**
     * 查看运输派单
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getTransportDispatch(Map<String, Object> map) {
        return omstruckOrderMapper.getTransportDispatch(map);
    }

    /**
     * 用于更新路段顺序号
     * @param map  orderId
     * @return
     */
    public int updateOrderLineSequence(Map<String, Object> map, Integer lineSequence){
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        omstruckOrder.setOrderId(DataChange.dataToBigDecimal(map.get("orderId")));
        omstruckOrder.setOrderLineSequence(new BigDecimal(lineSequence));
        return omstruckOrderMapper.updateByPrimaryKeySelective(omstruckOrder);
    }
}
