package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.OmstruckOrderMapper;
import com.steerinfo.dil.mapper.TmstruckEnfactoryResultMapper;
import com.steerinfo.dil.mapper.TmstruckLeaveFactoryResultMapper;
import com.steerinfo.dil.model.OmstruckOrder;
import com.steerinfo.dil.model.TmstruckEnfactoryResult;
import com.steerinfo.dil.service.ITmstruckEnfactoryResultService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * TmstruckEnfactoryResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-08 06:23
 * 类描述
 * 修订历史：
 * 日期：2021-09-08
 * 作者：generator
 * 参考：
 * 描述：TmstruckEnfactoryResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckEnfactoryResultService")
public class TmstruckEnfactoryResultServiceImpl implements ITmstruckEnfactoryResultService {

    @Autowired
    private TmstruckEnfactoryResultMapper tmstruckEnfactoryResultMapper;

    @Autowired
    private OmstruckOrderServiceImpl omstruckOrderService;

    @Autowired
    private TmstruckMeasureCommissionServiceImpl tmstruckMeasureCommissionService;

    /**
     * 获取所有的进厂实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllEnFactoryResult(Map<String, Object> map) {
        return tmstruckEnfactoryResultMapper.getAllEnFactoryResult(map);
    }

    /**
     * 派单时新增进厂实绩
     * 总实绩Id、线路起点
     * @param map
     * @return
     */
    @Override
    public int addEnFactoryResult(Map<String, Object> map) {
        TmstruckEnfactoryResult tmstruckEnfactoryResult = new TmstruckEnfactoryResult();
        //生成进厂作业主键Id
        //添加主键
        tmstruckEnfactoryResult.setResultId(tmstruckEnfactoryResultMapper.selectMaxId());
        //添加门岗ID 路线起点就是进厂门岗
        tmstruckEnfactoryResult.setGatepostId(DataChange.dataToBigDecimal(map.get("lineStartNodeId")));
        //添加实绩总表Id
        tmstruckEnfactoryResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        return tmstruckEnfactoryResultMapper.insertSelective(tmstruckEnfactoryResult);
    }

    /**
     * 使用PDA方式进厂
     * 运输订单号
     * (操作人员主键ID)
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int updateEnFactoryByPDA(Map<String, Object> map) {
        TmstruckEnfactoryResult tmstruckEnfactoryResult = new TmstruckEnfactoryResult();
        String orderNumber = map.get("orderNumber").toString();
        //获取进厂实绩id
        map.putAll(tmstruckEnfactoryResultMapper.selectEnFactoryResultId(orderNumber));
        tmstruckEnfactoryResult.setResultId(DataChange.dataToBigDecimal(map.get("resultId")));
        //添加进厂时间
        tmstruckEnfactoryResult.setResultEntryGateTime(new Date());
        tmstruckEnfactoryResult.setInsertTime(new Date());
        tmstruckEnfactoryResult.setResultEntryMode("手动抬杠");
        tmstruckEnfactoryResult.setInsertUsername("admin");
        int i = tmstruckEnfactoryResultMapper.updateByPrimaryKeySelective(tmstruckEnfactoryResult);
        //更新订单路段顺序号
        int i1 = omstruckOrderService.updateOrderLineSequence(map, 1);
        //新增一条计毛委托
        //进厂完成后新增一条计毛委托 类型 1
        BigDecimal resultTotalId = DataChange.dataToBigDecimal(map.get("resultTotalId"));
        int i2 = tmstruckMeasureCommissionService.addMeasureCommission(resultTotalId, 1);
        return i + i1 +i2;
    }

    /**
     * 使用门禁系统进厂
     * @param map
     * @return
     */
    @Override
    public int updateEnFactoryByMenJin(Map<String, Object> map) {
        TmstruckEnfactoryResult tmstruckEnfactoryResult=new TmstruckEnfactoryResult();
        String orderNumber = map.get("orderNumber").toString();
        //获取进厂实绩id
        map.putAll(tmstruckEnfactoryResultMapper.selectEnFactoryResultId(orderNumber));
        tmstruckEnfactoryResult.setResultId(DataChange.dataToBigDecimal(map.get("resultId")));
        //进门方式
        tmstruckEnfactoryResult.setResultEntryMode("自动抬杆");
        tmstruckEnfactoryResult.setInsertTime(new Date());
        tmstruckEnfactoryResult.setInsertUsername("admin");
        int i = tmstruckEnfactoryResultMapper.updateByPrimaryKeySelective(tmstruckEnfactoryResult);
        //更新路段顺序号
        int i1 = omstruckOrderService.updateOrderLineSequence(map, 1);
        //进厂完成后新增一条计毛委托 类型 1
        BigDecimal resultTotalId = DataChange.dataToBigDecimal(map.get("resultTotalId"));
        int i2 = tmstruckMeasureCommissionService.addMeasureCommission(resultTotalId, 1);
        return i + i1 +i2;
    }

    /**
     * 通过运输订单号查找排队顺序号
     * @param orderNumber
     * @return
     */
    @Override
    public Integer selectNodeNumber(String orderNumber) {
        return tmstruckEnfactoryResultMapper.selectNodeNumber(orderNumber);
    }
}
