package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.feign.BmsShipFeign;
import com.steerinfo.dil.feign.BmsTruckFeign;
import com.steerinfo.dil.mapper.OmstruckOrderMapper;
import com.steerinfo.dil.mapper.TmstruckEnfactoryResultMapper;
import com.steerinfo.dil.mapper.TmstruckLeaveFactoryResultMapper;
import com.steerinfo.dil.model.OmstruckOrder;
import com.steerinfo.dil.model.TmstruckLeaveFactoryResult;
import com.steerinfo.dil.service.ITmstruckLeaveFactoryResultService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * TmstruckLeaveFactoryResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-11 10:32
 * 类描述
 * 修订历史：
 * 日期：2021-09-11
 * 作者：generator
 * 参考：
 * 描述：TmstruckLeaveFactoryResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckLeaveFactoryResultService")
public class TmstruckLeaveFactoryResultServiceImpl implements ITmstruckLeaveFactoryResultService {

    @Autowired
    private TmstruckLeaveFactoryResultMapper tmstruckLeaveFactoryResultMapper;

    @Autowired
    private TmstruckEnfactoryResultMapper tmstruckEnfactoryResultMapper;

    @Autowired
    private OmstruckOrderServiceImpl omstruckOrderService;

    @Autowired
    private OmstruckOrderMapper omstruckOrderMapper;

    @Autowired
    private BmsShipFeign bmsShipFeign;

    @Autowired
    private BmsTruckFeign bmsTruckFeign;
    /**
     * 新增出厂实绩  需要采集的数据
     *  运输订单号
     *  车牌号
     *  门岗名称
     *  出厂时间
     *  出厂方式
     *  车厢抓拍图片
     * @param map
     * @return
     */
    @Override
    public int addLeaveFactoryResult(Map<String, Object> map) {
        TmstruckLeaveFactoryResult tmstruckLeaveFactoryResult = new TmstruckLeaveFactoryResult();
        //查询最大主键ID
        tmstruckLeaveFactoryResult.setId(tmstruckLeaveFactoryResultMapper.selectMaxId());
        //获取运输订单号
        String orderNumber = (String) map.get("orderNumber");
        //获取车牌号
        String capacityNumber = (String) map.get("capacityNumber");
        HashMap<String, Object> map1 = new HashMap<>();
        map1.put("orderNumber", orderNumber);
        map1.put("capacityNumber", capacityNumber);
        //通过运输订单号、车牌号查询 总实绩ID
        Integer totalId = tmstruckEnfactoryResultMapper.selectOrderIdByCpAndOn(map1);
        tmstruckLeaveFactoryResult.setResultTotalId(new BigDecimal(totalId));
        //获取门岗名称
        String gatepostName = (String) map.get("gatepostName");
        Integer gatepostId = tmstruckEnfactoryResultMapper.selectGatepostIdByGatepostName(gatepostName);
        tmstruckLeaveFactoryResult.setGatepostId(new BigDecimal(gatepostId));
        //获取出厂时间
        long resultOutGateTime = (long)map.get("resultOutGateTime");
        tmstruckLeaveFactoryResult.setResultOutGateTime(new Date(resultOutGateTime));
        //获取出厂方式
        String resultOutMode = (String) map.get("resultOutMode");
        tmstruckLeaveFactoryResult.setResultOutMode(resultOutMode);

        tmstruckLeaveFactoryResult.setInsertTime(new Date());
        tmstruckLeaveFactoryResult.setInsertUsername("admin");
        return tmstruckLeaveFactoryResultMapper.insertSelective(tmstruckLeaveFactoryResult);
    }

    /**
     * 查询所有的出厂实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getLeaveFactoryResult(Map<String, Object> map) {
        return tmstruckLeaveFactoryResultMapper.getLeaveFactoryResult(map);
    }

    /**
     * 查看运输线路是否为出厂
     * @param orderNumber
     * @return
     */
    @Override
    public Integer selectTransportRoute(String orderNumber) {
        return tmstruckLeaveFactoryResultMapper.selectTransportRoute(orderNumber);
    }

    /**
     * 通过运输订单号获取 车牌号 和 门岗
     * @param orderNumber
     * @return
     */
    @Override
    public Map<String, Object> selectLeaveFactory(String orderNumber) {
        return tmstruckLeaveFactoryResultMapper.selectCnAndGnByOrderNumber(orderNumber);
    }

    /**
     * 派单后新增出厂实绩
     * 总实绩ID、线路终点
     * @param map
     * @return
     */
    @Override
    public int addLeaveFactory(Map<String, Object> map) {
        TmstruckLeaveFactoryResult tmstruckLeaveFactoryResult = new TmstruckLeaveFactoryResult();
        //添加主键
        tmstruckLeaveFactoryResult.setId(tmstruckLeaveFactoryResultMapper.selectMaxId());
        //添加总实绩ID
        tmstruckLeaveFactoryResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        //添加门岗ID 出厂门岗为线路的终点
        tmstruckLeaveFactoryResult.setGatepostId(DataChange.dataToBigDecimal(map.get("lineEndNodeId")));
        return tmstruckLeaveFactoryResultMapper.insertSelective(tmstruckLeaveFactoryResult);
    }

    /**
     * 通过PDA进行更新实绩
     * 运输订单号
     * @param mapValue
     * @return
     */
    @Override
    @Transactional
    public int updateLeaveFactoryByPDA(Map<String, Object> mapValue){
        //获取订单号
        String orderNumber = mapValue.get("orderNumber").toString();
        //根据订单号获取实绩id
        mapValue.putAll(tmstruckLeaveFactoryResultMapper.selectResultId(orderNumber));
        TmstruckLeaveFactoryResult tmstruckLeaveFactoryResult = new TmstruckLeaveFactoryResult();
        tmstruckLeaveFactoryResult.setResultId(DataChange.dataToBigDecimal(mapValue.get("resultId")));
        //设置常规字段
        tmstruckLeaveFactoryResult.setResultOutGateTime(new Date());
        tmstruckLeaveFactoryResult.setResultOutMode("手动抬杠");
        tmstruckLeaveFactoryResult.setInsertUsername("admin");
        tmstruckLeaveFactoryResult.setInsertTime(new Date());
        int i = tmstruckLeaveFactoryResultMapper.updateByPrimaryKeySelective(tmstruckLeaveFactoryResult);
        //更改路段顺序号为 5 ：出厂
        int i1 = omstruckOrderService.updateOrderLineSequence(mapValue, 5);
        //将运力置为空闲状态
        mapValue.put("capacityStatus", 0);
        int i3 = omstruckOrderMapper.updateCapacityByCarIdOrNum(mapValue);
        //将订单状态改为已结束 2
        OmstruckOrder omstruckOrder = new OmstruckOrder();
        BigDecimal orderId = DataChange.dataToBigDecimal(mapValue.get("orderId"));
        omstruckOrder.setOrderId(orderId);
        omstruckOrder.setOrderStatus(new BigDecimal(2));
        int i2 = omstruckOrderMapper.updateByPrimaryKeySelective(omstruckOrder);
        //通过feign调用新增详单

        try {
            bmsShipFeign.addDetailsOrder(tmstruckLeaveFactoryResult.getResultId()); // 出厂实绩Id
            bmsTruckFeign.addDetailsOrder(orderId);  // 运单ID
        } catch (Exception e) {
            e.printStackTrace();
        }
        return i + i1 + i2 + i3;
    }

    /**
     * 通过门岗系统进行更新实绩
     * @param mapValue
     * @return
     */
    @Override
    public int updateLeaveFactoryByMenGang(Map<String, Object> mapValue) {
        //获取订单号
        String orderNumber=mapValue.get("orderNumber").toString();
        //根据订单号获取实绩id
        mapValue.putAll(tmstruckLeaveFactoryResultMapper.selectResultId(orderNumber));
        TmstruckLeaveFactoryResult tmstruckLeaveFactoryResult = new TmstruckLeaveFactoryResult();
        tmstruckLeaveFactoryResult.setResultId(DataChange.dataToBigDecimal(mapValue.get("resultId")));
        //设置常规字段
        tmstruckLeaveFactoryResult.setResultOutGateTime(new Date());
        String  ResultLeaveModel=mapValue.get("ResultLeaveModel").toString();
        tmstruckLeaveFactoryResult.setResultOutMode(ResultLeaveModel);
        tmstruckLeaveFactoryResult.setInsertUsername("admin");
        tmstruckLeaveFactoryResult.setInsertTime(new Date());
        int i = tmstruckLeaveFactoryResultMapper.updateByPrimaryKeySelective(tmstruckLeaveFactoryResult);
        //更改路段顺序号为 5 ：出厂
        int i1 = omstruckOrderService.updateOrderLineSequence(mapValue, 5);
        return i + i1;
    }
}
