package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.TmstruckReceiptResultMapper;
import com.steerinfo.dil.mapper.TmstruckUnloadResultMapper;
import com.steerinfo.dil.mapper.TmstruckWeightResultMapper;
import com.steerinfo.dil.model.TmstruckReceiptResult;
import com.steerinfo.dil.model.TmstruckUnloadResult;
import com.steerinfo.dil.service.ITmstruckUnloadResultService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * TmstruckUnloadResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-09 07:58
 * 类描述
 * 修订历史：
 * 日期：2021-09-09
 * 作者：generator
 * 参考：
 * 描述：TmstruckUnloadResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckUnloadResultService")
public class TmstruckUnloadResultServiceImpl implements ITmstruckUnloadResultService {

    @Autowired
    private TmstruckUnloadResultMapper tmstruckUnloadResultMapper;

    @Autowired
    private TmstruckReceiptResultMapper tmstruckReceiptResultMapper;

    @Autowired
    private OmstruckOrderServiceImpl omstruckOrderService;

    @Autowired
    private TmstruckMeasureCommissionServiceImpl tmstruckMeasureCommissionService;


    /**
     * 查询所有卸货实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getUnloadResult(Map<String, Object> map) {
        return tmstruckUnloadResultMapper.getUnloadResult(map);
    }

    /**
     * 新增卸货实绩
     * 总实绩ID、卸货点
     * @param map
     * @return
     */
    @Override
    public int addUnloadResult(Map<String, Object> map) {
        TmstruckUnloadResult tmstruckUnloadResult = new TmstruckUnloadResult();
        //生成主键ID
        BigDecimal maxId = tmstruckUnloadResultMapper.selectMaxId();
        tmstruckUnloadResult.setId(maxId);
        //生成卸货实绩编号
        int id = maxId.intValue();
        StringBuilder resultNumber = new StringBuilder("DW");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
        String timeFormat = sdf.format(new Date());
        resultNumber.append(timeFormat);
        resultNumber.append(id < 10 ? "0000" + id : id < 100 ? "000" + id : id < 1000 ? "00" +id : id < 10000 ? "0" + id : String.valueOf(id));
        tmstruckUnloadResult.setResultNumber(resultNumber.toString());
        //插入实绩总表ID
        tmstruckUnloadResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        //插入卸货点
        tmstruckUnloadResult.setResultUnloadPlaceId(DataChange.dataToBigDecimal(map.get("resultUnloadPlaceId")));
        //新增卸车实绩
        int i = tmstruckUnloadResultMapper.insertSelective(tmstruckUnloadResult);
        return i;
    }

    /**
     * 更新卸货实绩内容 扫码人员扫码后获取信息传输数据给接口
     *  运输订单号
     *  卸货时间
     *  卸货地点
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int updateUnloadResult(Map<String, Object> map) {
        TmstruckUnloadResult tmstruckUnloadResult = new TmstruckUnloadResult();
        //通过获取的运输订单号查询实绩总表id
        map.putAll(tmstruckUnloadResultMapper.selectTotalIdByOrderNo((String) map.get("orderNumber")));
        //得到实绩id
        tmstruckUnloadResult.setResultId(DataChange.dataToBigDecimal(map.get("resultId")));
        //通过卸货地点（仓库）查询仓库ID
        BigDecimal resultUnloadPlaceId = tmstruckUnloadResultMapper.getWarehouseIdByWName((String) map.get("warehouseName"));
        // 修改卸货点
        if(resultUnloadPlaceId != null){
            tmstruckUnloadResult.setResultUnloadPlaceId(resultUnloadPlaceId);
        }
        //获得卸货开始时间
        tmstruckUnloadResult.setResultStartTime(new Date((long) map.get("unloadTime")));
        //扫码后确认卸货所以卸货状态码为  1 同意
        tmstruckUnloadResult.setUnloadStatus(new BigDecimal(1));
        tmstruckUnloadResult.setInsertTime(new Date());
        tmstruckUnloadResult.setInsertUsername("admin");
        //新增卸车实绩内容
        int i = tmstruckUnloadResultMapper.updateByPrimaryKeySelective(tmstruckUnloadResult);
        //卸货完成后新增一条计皮委托 类型 0
        BigDecimal resultTotalId = DataChange.dataToBigDecimal(map.get("resultTotalId"));
        int i1 = tmstruckMeasureCommissionService.addMeasureCommission(resultTotalId, 0);

        //新增完卸车实绩后新增一条收货实绩
        TmstruckReceiptResult tmstruckReceiptResult = new TmstruckReceiptResult();
        tmstruckReceiptResult.setId(tmstruckReceiptResultMapper.selectMaxId());//添加主键
        tmstruckReceiptResult.setResultTotalId(resultTotalId);//添加总实绩Id
        tmstruckReceiptResult.setStatus(new BigDecimal(0)); // 卸货完成后添加的为虚拟收货实绩
        int i2 = tmstruckReceiptResultMapper.insertSelective(tmstruckReceiptResult);

        //更新订单路段顺序号
        int i3 = omstruckOrderService.updateOrderLineSequence(map, 3);
        return i + i1 + i2 +i3;
    }
}
