package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.OmstruckOrderMapper;
import com.steerinfo.dil.mapper.TmstruckReceiptResultMapper;
import com.steerinfo.dil.mapper.TmstruckWeightResultMapper;
import com.steerinfo.dil.model.OmstruckOrder;
import com.steerinfo.dil.model.TmstruckReceiptResult;
import com.steerinfo.dil.model.TmstruckWeightResult;
import com.steerinfo.dil.service.ITmstruckWeightResultService;
import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * TmstruckWeightResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-09-09 02:21
 * 类描述
 * 修订历史：
 * 日期：2021-09-09
 * 作者：generator
 * 参考：
 * 描述：TmstruckWeightResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckWeightResultService")
public class TmstruckWeightResultServiceImpl implements ITmstruckWeightResultService {

    @Autowired
    private TmstruckWeightResultMapper tmstruckWeightResultMapper;

    @Autowired
    private TmstruckReceiptResultMapper tmstruckReceiptResultMapper;

    @Autowired
    private OmstruckOrderServiceImpl omstruckOrderService;

    /**
     * 派单时新增计重实绩
     * 总实绩ID、计皮点、计毛点
     * @return
     */
    @Override
    public int addWeightResult(Map<String, Object> map) {
        TmstruckWeightResult tmstruckWeightResult = new TmstruckWeightResult();
        //生成最大主键ID
        tmstruckWeightResult.setWeightTaskResultId(tmstruckWeightResultMapper.selectMaxId());
        tmstruckWeightResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        // 添加计皮计毛作业点ID
        tmstruckWeightResult.setResultGrossPlaceId(DataChange.dataToBigDecimal(map.get("truckCalculateIdMao")));
        tmstruckWeightResult.setResultTarePlaceId(DataChange.dataToBigDecimal(map.get("truckCalculateIdPi")));
        return tmstruckWeightResultMapper.insertSelective(tmstruckWeightResult);
    }

    /**
     * 更新计毛实绩 采集需要的数据
     *  运输订单号
     *  汽车衡编号
     *  //物资名称
     *  毛重
     *  毛重时间
     *  磅单号
     * @return
     */
    @Override
    @Transactional
    public int addJiMaoResult(Map<String, Object> map) {
        TmstruckWeightResult tmstruckWeightResult = new TmstruckWeightResult();
        double resultGrossWeight = (double) map.get("resultGrossWeight");
        //通过传来的运输订单号查询总实绩ID 计量实绩Id
        map.putAll(tmstruckWeightResultMapper.selectTotalIdByOrderNo((String) map.get("orderNumber")));
        BigDecimal resultId = DataChange.dataToBigDecimal(map.get("weightTaskResultId"));
        tmstruckWeightResult.setWeightTaskResultId(resultId);
        //添加毛重
        DecimalFormat df =  new DecimalFormat("0.00");
        String grossWeight = df.format(resultGrossWeight);
        tmstruckWeightResult.setResultGrossWeight(new BigDecimal(grossWeight));
        //通过汽车衡编号查询汽车衡ID
        Integer calculateId = tmstruckWeightResultMapper.selectTruckCalculateId((String) map.get("truckCalculateNumber"));
        //添加计毛汽车衡
        if(calculateId != null){
            tmstruckWeightResult.setResultGrossPlaceId(new BigDecimal(calculateId));
        }
        //添加磅单号
        tmstruckWeightResult.setResultPoundNo((String) map.get("resultPoundNo"));
        //添加毛重时间
        long resultGrossWeightTime = (long) map.get("resultGrossWeightTime");
        tmstruckWeightResult.setResultGrossWeightTime(new Date(resultGrossWeightTime));
        tmstruckWeightResult.setInsertTime(new Date());
        tmstruckWeightResult.setInsertUsername("admin");
        tmstruckWeightResult.setInsertUpdateRemark("无");
        int i = tmstruckWeightResultMapper.updateByPrimaryKeySelective(tmstruckWeightResult);
        //更新订单路段顺序号
        int i2 = omstruckOrderService.updateOrderLineSequence(map, 2);
        return i +i2;
    }

    /**
     * 查询所有计毛实绩
     * @param map
     * @return
     */

    @Override
    public List<Map<String, Object>> getAllJiMaoResult(Map<String, Object> map) {
        return tmstruckWeightResultMapper.getAllJiMaoResult(map);
    }

    /**
     * 查询所有计皮实绩
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllJiPiResult(Map<String, Object> map) {
        return tmstruckWeightResultMapper.getAllJiPiResult(map);
    }

    /**
     * 更新计皮实绩  需要采集的数据
     * 运输订单号
     * 汽车衡编号
     * 皮重
     * 皮重时间
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public int addJiPiResult(Map<String, Object> map) {
        TmstruckWeightResult tmstruckWeightResult = new TmstruckWeightResult();
        //通过传来的运输订单号查询总实绩ID
        map.putAll(tmstruckWeightResultMapper.selectTotalIdByOrderNo((String) map.get("orderNumber")));
        BigDecimal resultId = (BigDecimal) map.get("weightTaskResultId");
        tmstruckWeightResult.setWeightTaskResultId(resultId);
        //添加皮重
        DecimalFormat df =  new DecimalFormat("0.00");
        String resultTareWeight = df.format((double) map.get("resultTareWeight"));
        tmstruckWeightResult.setResultTareWeight(new BigDecimal(resultTareWeight));
        //添加皮重时间
        long resultTareWeightTime = (long) map.get("resultTareWeightTime");
        tmstruckWeightResult.setResultTareWeightTime(new Date(resultTareWeightTime));
        //计算净重  先获取毛重
        double netWeight = ((BigDecimal) map.get("resultGrossWeight")).doubleValue() - ((Double) map.get("resultTareWeight"));
        String resultNetWeight = df.format(netWeight);
        tmstruckWeightResult.setResultNetWeight(new BigDecimal(resultNetWeight));
        //通过汽车衡编号查询汽车衡ID
        Integer calculateId = tmstruckWeightResultMapper.selectTruckCalculateId((String) map.get("truckCalculateNumber"));
        //添加计皮汽车衡
        if(calculateId != null){
            tmstruckWeightResult.setResultTarePlaceId(new BigDecimal(calculateId));
        }
        tmstruckWeightResult.setUpdateTime(new Date());
        tmstruckWeightResult.setUpdateUsername("admin");
        int i = tmstruckWeightResultMapper.updateByPrimaryKeySelective(tmstruckWeightResult);
        //计皮作业完成后自动完成签收
        TmstruckReceiptResult tmstruckReceiptResult = new TmstruckReceiptResult();
        tmstruckReceiptResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        tmstruckReceiptResult.setStatus(new BigDecimal(1));
        tmstruckReceiptResult.setInsertUsername("admin");
        tmstruckReceiptResult.setInsertTime(new Date());
        int i1 = tmstruckReceiptResultMapper.updateByTotalResultId(tmstruckReceiptResult);
        //更新订单路段顺序号
        int i2 = omstruckOrderService.updateOrderLineSequence(map, 4);
        return i + i1 + i2;
    }
}
